# Driver for official MicroPython LCD160CR display
# MIT license; Copyright (c) 2017 Damien P. George

from micropython import const
import machine
from utime import sleep_ms
from ustruct import calcsize, pack_into
import errno

# for set_orient
PORTRAIT = const(0)
LANDSCAPE = const(1)
PORTRAIT_UPSIDEDOWN = const(2)
LANDSCAPE_UPSIDEDOWN = const(3)

# for set_startup_deco; can be or'd
STARTUP_DECO_NONE = const(0)
STARTUP_DECO_MLOGO = const(1)
STARTUP_DECO_INFO = const(2)

_uart_baud_table = {
    2400: 0,
    4800: 1,
    9600: 2,
    19200: 3,
    38400: 4,
    57600: 5,
    115200: 6,
    230400: 7,
    460800: 8,
}


class LCD160CR:
    def __init__(self, connect=None, *, pwr=None, i2c=None, spi=None, i2c_addr=98):
        if connect in ("X", "Y", "XY", "YX"):
            i = connect[-1]
            j = connect[0]
            y = j + "4"
        elif connect == "C":
            i = 2
            j = 2
            y = "A7"
        else:
            if pwr is None or i2c is None or spi is None:
                raise ValueError('must specify valid "connect" or all of "pwr", "i2c" and "spi"')

        if pwr is None:
            pwr = machine.Pin(y, machine.Pin.OUT)
        if i2c is None:
            i2c = machine.I2C(i, freq=1000000)
        if spi is None:
            spi = machine.SPI(j, baudrate=13500000, polarity=0, phase=0)

        if not pwr.value():
            pwr(1)
            sleep_ms(10)
        # else:
        # alread have power
        # lets be optimistic...

        # set connections
        self.pwr = pwr
        self.i2c = i2c
        self.spi = spi
        self.i2c_addr = i2c_addr

        # create temp buffers and memoryviews
        self.buf16 = bytearray(16)
        self.buf19 = bytearray(19)
        self.buf = [None] * 10
        for i in range(1, 10):
            self.buf[i] = memoryview(self.buf16)[0:i]
        self.buf1 = self.buf[1]
        self.array4 = [0, 0, 0, 0]

        # set default orientation and window
        self.set_orient(PORTRAIT)
        self._fcmd2b("<BBBBBB", 0x76, 0, 0, self.w, self.h)  # viewport 'v'
        self._fcmd2b("<BBBBBB", 0x79, 0, 0, self.w, self.h)  # window 'y'

    def _send(self, cmd):
        i = self.i2c.writeto(self.i2c_addr, cmd)
        if i == len(cmd):
            return
        cmd = memoryview(cmd)
        n = len(cmd)
        while True:
            i += self.i2c.writeto(self.i2c_addr, cmd[i:])
            if i == n:
                return
            sleep_ms(10)

    def _fcmd2(self, fmt, a0, a1=0, a2=0):
        buf = self.buf[calcsize(fmt)]
        pack_into(fmt, buf, 0, 2, a0, a1, a2)
        self._send(buf)

    def _fcmd2b(self, fmt, a0, a1, a2, a3, a4=0):
        buf = self.buf[calcsize(fmt)]
        pack_into(fmt, buf, 0, 2, a0, a1, a2, a3, a4)
        self._send(buf)

    def _waitfor(self, n, buf):
        t = 5000
        while t:
            self.i2c.readfrom_into(self.i2c_addr, self.buf1)
            if self.buf1[0] >= n:
                self.i2c.readfrom_into(self.i2c_addr, buf)
                return
            t -= 1
            sleep_ms(1)
        raise OSError(errno.ETIMEDOUT)

    def oflush(self, n=255):
        t = 5000
        while t:
            self.i2c.readfrom_into(self.i2c_addr + 1, self.buf1)
            r = self.buf1[0]
            if r >= n:
                return
            t -= 1
            machine.idle()
        raise OSError(errno.ETIMEDOUT)

    def iflush(self):
        t = 5000
        while t:
            self.i2c.readfrom_into(self.i2c_addr, self.buf16)
            if self.buf16[0] == 0:
                return
            t -= 1
            sleep_ms(1)
        raise OSError(errno.ETIMEDOUT)

    #### MISC METHODS ####

    @staticmethod
    def rgb(r, g, b):
        return ((b & 0xF8) << 8) | ((g & 0xFC) << 3) | (r >> 3)

    @staticmethod
    def clip_line(c, w, h):
        while True:
            ca = ce = 0
            if c[1] < 0:
                ca |= 8
            elif c[1] > h:
                ca |= 4
            if c[0] < 0:
                ca |= 1
            elif c[0] > w:
                ca |= 2
            if c[3] < 0:
                ce |= 8
            elif c[3] > h:
                ce |= 4
            if c[2] < 0:
                ce |= 1
            elif c[2] > w:
                ce |= 2
            if ca & ce:
                return False
            elif ca | ce:
                ca |= ce
                if ca & 1:
                    if c[2] < c[0]:
                        c[0], c[2] = c[2], c[0]
                        c[1], c[3] = c[3], c[1]
                    c[1] += ((-c[0]) * (c[3] - c[1])) // (c[2] - c[0])
                    c[0] = 0
                elif ca & 2:
                    if c[2] < c[0]:
                        c[0], c[2] = c[2], c[0]
                        c[1], c[3] = c[3], c[1]
                    c[3] += ((w - 1 - c[2]) * (c[3] - c[1])) // (c[2] - c[0])
                    c[2] = w - 1
                elif ca & 4:
                    if c[0] == c[2]:
                        if c[1] >= h:
                            c[1] = h - 1
                        if c[3] >= h:
                            c[3] = h - 1
                    else:
                        if c[3] < c[1]:
                            c[0], c[2] = c[2], c[0]
                            c[1], c[3] = c[3], c[1]
                        c[2] += ((h - 1 - c[3]) * (c[2] - c[0])) // (c[3] - c[1])
                        c[3] = h - 1
                else:
                    if c[0] == c[2]:
                        if c[1] < 0:
                            c[1] = 0
                        if c[3] < 0:
                            c[3] = 0
                    else:
                        if c[3] < c[1]:
                            c[0], c[2] = c[2], c[0]
                            c[1], c[3] = c[3], c[1]
                        c[0] += ((-c[1]) * (c[2] - c[0])) // (c[3] - c[1])
                        c[1] = 0
            else:
                return True

    #### SETUP COMMANDS ####

    def set_power(self, on):
        self.pwr(on)
        sleep_ms(15)

    def set_orient(self, orient):
        self._fcmd2("<BBB", 0x14, (orient & 3) + 4)
        # update width and height variables
        self.iflush()
        self._send(b"\x02g0")
        self._waitfor(4, self.buf[5])
        self.w = self.buf[5][1]
        self.h = self.buf[5][2]

    def set_brightness(self, value):
        self._fcmd2("<BBB", 0x16, value)

    def set_i2c_addr(self, addr):
        # 0x0e set i2c addr
        if addr & 3:
            raise ValueError("must specify mod 4 aligned address")
        self._fcmd2("<BBW", 0x0E, 0x433249 | (addr << 24))

    def set_uart_baudrate(self, baudrate):
        try:
            baudrate = _uart_baud_table[baudrate]
        except KeyError:
            raise ValueError("invalid baudrate")
        self._fcmd2("<BBB", 0x18, baudrate)

    def set_startup_deco(self, value):
        self._fcmd2("<BBB", 0x19, value)

    def save_to_flash(self):
        self._send(b"\x02fn")

    #### PIXEL ACCESS ####

    def set_pixel(self, x, y, c):
        self._fcmd2b("<BBBBH", 0x41, x, y, c)

    def get_pixel(self, x, y):
        self._fcmd2("<BBBB", 0x61, x, y)
        t = 1000
        while t:
            self.i2c.readfrom_into(self.i2c_addr, self.buf1)
            if self.buf1[0] >= 2:
                self.i2c.readfrom_into(self.i2c_addr, self.buf[3])
                return self.buf[3][1] | self.buf[3][2] << 8
            t -= 1
            sleep_ms(1)
        raise OSError(errno.ETIMEDOUT)

    def get_line(self, x, y, buf):
        l = len(buf) // 2
        self._fcmd2b("<BBBBB", 0x10, l, x, y)
        l *= 2
        t = 1000
        while t:
            self.i2c.readfrom_into(self.i2c_addr, self.buf1)
            if self.buf1[0] >= l:
                self.i2c.readfrom_into(self.i2c_addr, buf)
                return
            t -= 1
            sleep_ms(1)
        raise OSError(errno.ETIMEDOUT)

    def screen_dump(self, buf, x=0, y=0, w=None, h=None):
        if w is None:
            w = self.w - x
        if h is None:
            h = self.h - y
        if w <= 127:
            line = bytearray(2 * w + 1)
            line2 = None
        else:
            # split line if more than 254 bytes needed
            buflen = (w + 1) // 2
            line = bytearray(2 * buflen + 1)
            line2 = memoryview(line)[: 2 * (w - buflen) + 1]
        for i in range(min(len(buf) // (2 * w), h)):
            ix = i * w * 2
            self.get_line(x, y + i, line)
            buf[ix : ix + len(line) - 1] = memoryview(line)[1:]
            ix += len(line) - 1
            if line2:
                self.get_line(x + buflen, y + i, line2)
                buf[ix : ix + len(line2) - 1] = memoryview(line2)[1:]
                ix += len(line2) - 1

    def screen_load(self, buf):
        l = self.w * self.h * 2 + 2
        self._fcmd2b("<BBHBBB", 0x70, l, 16, self.w, self.h)
        n = 0
        ar = memoryview(buf)
        while n < len(buf):
            if len(buf) - n >= 0x200:
                self._send(ar[n : n + 0x200])
                n += 0x200
            else:
                self._send(ar[n:])
                while n < self.w * self.h * 2:
                    self._send(b"\x00")
                    n += 1

    #### TEXT COMMANDS ####

    def set_pos(self, x, y):
        self._fcmd2("<BBBB", 0x58, x, y)

    def set_text_color(self, fg, bg):
        self._fcmd2("<BBHH", 0x63, fg, bg)

    def set_font(self, font, scale=0, bold=0, trans=0, scroll=0):
        self._fcmd2(
            "<BBBB",
            0x46,
            (scroll << 7) | (trans << 6) | ((font & 3) << 4) | (bold & 0xF),
            scale & 0xFF,
        )

    def write(self, s):
        # TODO: eventually check for room in LCD input queue
        self._send(s)

    #### PRIMITIVE DRAWING COMMANDS ####

    def set_pen(self, line, fill):
        self._fcmd2("<BBHH", 0x50, line, fill)

    def erase(self):
        self._send(b"\x02\x45")

    def dot(self, x, y):
        if 0 <= x < self.w and 0 <= y < self.h:
            self._fcmd2("<BBBB", 0x4B, x, y)

    def rect(self, x, y, w, h, cmd=0x72):
        if x + w <= 0 or y + h <= 0 or x >= self.w or y >= self.h:
            return
        elif x < 0 or y < 0:
            left = top = True
            if x < 0:
                left = False
                w += x
                x = 0
            if y < 0:
                top = False
                h += y
                y = 0
            if cmd == 0x51 or cmd == 0x72:
                # draw interior
                self._fcmd2b("<BBBBBB", 0x51, x, y, min(w, 255), min(h, 255))
            if cmd == 0x57 or cmd == 0x72:
                # draw outline
                if left:
                    self._fcmd2b("<BBBBBB", 0x57, x, y, 1, min(h, 255))
                if top:
                    self._fcmd2b("<BBBBBB", 0x57, x, y, min(w, 255), 1)
                if x + w < self.w:
                    self._fcmd2b("<BBBBBB", 0x57, x + w, y, 1, min(h, 255))
                if y + h < self.h:
                    self._fcmd2b("<BBBBBB", 0x57, x, y + h, min(w, 255), 1)
        else:
            self._fcmd2b("<BBBBBB", cmd, x, y, min(w, 255), min(h, 255))

    def rect_outline(self, x, y, w, h):
        self.rect(x, y, w, h, 0x57)

    def rect_interior(self, x, y, w, h):
        self.rect(x, y, w, h, 0x51)

    def line(self, x1, y1, x2, y2):
        ar4 = self.array4
        ar4[0] = x1
        ar4[1] = y1
        ar4[2] = x2
        ar4[3] = y2
        if self.clip_line(ar4, self.w, self.h):
            self._fcmd2b("<BBBBBB", 0x4C, ar4[0], ar4[1], ar4[2], ar4[3])

    def dot_no_clip(self, x, y):
        self._fcmd2("<BBBB", 0x4B, x, y)

    def rect_no_clip(self, x, y, w, h):
        self._fcmd2b("<BBBBBB", 0x72, x, y, w, h)

    def rect_outline_no_clip(self, x, y, w, h):
        self._fcmd2b("<BBBBBB", 0x57, x, y, w, h)

    def rect_interior_no_clip(self, x, y, w, h):
        self._fcmd2b("<BBBBBB", 0x51, x, y, w, h)

    def line_no_clip(self, x1, y1, x2, y2):
        self._fcmd2b("<BBBBBB", 0x4C, x1, y1, x2, y2)

    def poly_dot(self, data):
        if len(data) & 1:
            raise ValueError("must specify even number of bytes")
        self._fcmd2("<BBB", 0x71, len(data) // 2)
        self._send(data)

    def poly_line(self, data):
        if len(data) & 1:
            raise ValueError("must specify even number of bytes")
        self._fcmd2("<BBB", 0x78, len(data) // 2)
        self._send(data)

    #### TOUCH COMMANDS ####

    def touch_config(self, calib=False, save=False, irq=None):
        self._fcmd2("<BBBB", 0x7A, (irq is not None) << 2 | save << 1 | calib, bool(irq) << 7)

    def is_touched(self):
        self._send(b"\x02T")
        b = self.buf[4]
        self._waitfor(3, b)
        return b[1] >> 7 != 0

    def get_touch(self):
        self._send(b"\x02T")  # implicit LCD output flush
        b = self.buf[4]
        self._waitfor(3, b)
        return b[1] >> 7, b[2], b[3]

    #### ADVANCED COMMANDS ####

    def set_spi_win(self, x, y, w, h):
        pack_into(
            "<BBBHHHHHHHH", self.buf19, 0, 2, 0x55, 10, x, y, x + w - 1, y + h - 1, 0, 0, 0, 0xFFFF
        )
        self._send(self.buf19)

    def fast_spi(self, flush=True):
        self._send(b"\x02\x12")
        if flush:
            self.oflush()
        return self.spi

    def show_framebuf(self, buf):
        self.fast_spi().write(buf)

    def set_scroll(self, on):
        self._fcmd2("<BBB", 0x15, on)

    def set_scroll_win(self, win, x=-1, y=0, w=0, h=0, vec=0, pat=0, fill=0x07E0, color=0):
        pack_into("<BBBHHHHHHHH", self.buf19, 0, 2, 0x55, win, x, y, w, h, vec, pat, fill, color)
        self._send(self.buf19)

    def set_scroll_win_param(self, win, param, value):
        self._fcmd2b("<BBBBH", 0x75, win, param, value)

    def set_scroll_buf(self, s):
        l = len(s)
        if l > 32:
            raise ValueError("length must be 32 or less")
        self._fcmd2("<BBB", 0x11, l)
        self._send(s)

    def jpeg_start(self, l):
        if l > 0xFFFF:
            raise ValueError("length must be 65535 or less")
        self.oflush()
        self._fcmd2("<BBH", 0x6A, l)

    def jpeg_data(self, buf):
        self._send(buf)

    def jpeg(self, buf):
        self.jpeg_start(len(buf))
        self.jpeg_data(buf)

    def feed_wdt(self):
        self._send(b"\x02\x17")

    def reset(self):
        self._send(b"\x02Y\xef\xbe\xad\xde")
        sleep_ms(15)



# log the accelerometer values to a .csv-file on the SD-card

import pyb

accel = pyb.Accel()  # create object of accelerometer
blue = pyb.LED(4)  # create object of blue LED

# open file to write data - /sd/ is the SD-card, /flash/ the internal memory
log = open("/sd/log.csv", "w")

blue.on()  # turn on blue LED

# do 100 times (if the board is connected via USB, you can't write longer because the PC tries to open the filesystem which messes up your file.)
for i in range(100):
    t = pyb.millis()  # get time since reset
    x, y, z = accel.filtered_xyz()  # get acceleration data
    log.write("{},{},{},{}\n".format(t, x, y, z))  # write data to file

log.close()  # close file
blue.off()  # turn off LED



# Test scoping rules for assignment expression :=.

# Test that var is closed over (assigned to in the scope of scope0).
def scope0():
    any((var0 := i) for i in range(2))
    return var0


print("scope0")
print(scope0())
print(globals().get("var0", None))

# Test that var1 gets closed over correctly in the list comprehension.
def scope1():
    var1 = 0
    dummy1 = 1
    dummy2 = 1
    print([var1 := i for i in [0, 1] if i > var1])
    print(var1)


print("scope1")
scope1()
print(globals().get("var1", None))

# Test that var2 in the comprehension honours the global declaration.
def scope2():
    global var2
    print([var2 := i for i in range(2)])
    print(globals().get("var2", None))


print("scope2")
scope2()
print(globals().get("var2", None))

# Test that var1 in the comprehension remains local to inner1.
def scope3():
    global var3

    def inner3():
        print([var3 := i for i in range(2)])

    inner3()
    print(globals().get("var3", None))


print("scope3")
scope3()
print(globals().get("var3", None))

# Test that var4 in the comprehension honours the global declarations.
def scope4():
    global var4

    def inner4():
        global var4
        print([var4 := i for i in range(2)])

    inner4()
    print(var4)


print("scope4")
scope4()
print(globals().get("var4", None))

# Test that var5 in the comprehension honours the nonlocal declaration.
def scope5():
    def inner5():
        nonlocal var5
        print([var5 := i for i in range(2)])

    inner5()
    print(var5)
    var5 = 0  # force var5 to be a local to scope5


print("scope5")
scope5()
print(globals().get("var5", None))



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2017 Glenn Ruben Bakke
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE

import time
from machine import PWM, Pin


class Servo:
    def __init__(self, pin_name=""):
        if pin_name:
            self.pin = Pin(pin_name, mode=Pin.OUT, pull=Pin.PULL_DOWN)
        else:
            self.pin = Pin("P22", mode=Pin.OUT, pull=Pin.PULL_DOWN)

    def left(self):
        p = PWM(
            0, self.pin, freq=PWM.FREQ_125KHZ, pulse_width=105, period=2500, mode=PWM.MODE_HIGH_LOW
        )
        p.init()
        time.sleep_ms(200)
        p.deinit()

    def center(self):
        p = PWM(
            0, self.pin, freq=PWM.FREQ_125KHZ, pulse_width=188, period=2500, mode=PWM.MODE_HIGH_LOW
        )
        p.init()
        time.sleep_ms(200)
        p.deinit()

    def right(self):
        p = PWM(
            0, self.pin, freq=PWM.FREQ_125KHZ, pulse_width=275, period=2500, mode=PWM.MODE_HIGH_LOW
        )
        p.init()
        time.sleep_ms(200)
        p.deinit()



"""
wipy module test for the CC3200 based boards
"""

import os
import wipy

mch = os.uname().machine
if not "LaunchPad" in mch and not "WiPy" in mch:
    raise Exception("Board not supported!")

print(wipy.heartbeat() == True)
wipy.heartbeat(False)
print(wipy.heartbeat() == False)
wipy.heartbeat(True)
print(wipy.heartbeat() == True)

try:
    wipy.heartbeat(True, 1)
except:
    print("Exception")



# Test ssl with key/cert passed in

try:
    import io
    import ssl
except ImportError:
    print("SKIP")
    raise SystemExit

# Only mbedTLS supports PKCS8 keys.
if not hasattr(ssl, "MBEDTLS_VERSION"):
    print("SKIP")
    raise SystemExit

# This particular key was generated with:
# $ openssl genrsa -out key.pem 512
# $ openssl rsa -in key.pem -outform DER -out key.der
keypkcs8 = b'0\x82\x01U\x02\x01\x000\r\x06\t*\x86H\x86\xf7\r\x01\x01\x01\x05\x00\x04\x82\x01?0\x82\x01;\x02\x01\x00\x02A\x00\xae\x07\xbb\xbf \x8eL\\ G\x91\x1b\xc45\x89\x96\x0c\xca\x94\xa5\xc2%F\x84z\xea\xb2Y\xdd\x943\x1a\xe0\x1e\xfe%\xd3\xbc\x85\xf0\x05\x8eI\xca\x83\x8b\x86\xfd\x15P,\xa9t8\xc2s\x11\x7fk*[[w\xdd\x02\x03\x01\x00\x01\x02@?\x02-\xc7\x95Q\xef\xf3\xaa\xfc>]\x7f<\xa3\x03\xe0\xbd\xb7\xf4\x0b\xc6\xd31D\x0f\x95cT\x06D9\xed\xf0X\xa6\x18\xb4\xe7\x96\x8f\x91\xd6\xd7\xbfR\\V\xfd\x0e{\xd3\xa4e\xb4M\x01\xbb\xdf\xf1\xf0\xbcEA\x02!\x00\xd5\xd7f\xcc\x84\xf5Y&\xa7]\x00\x91\xc9\x8e\xb0\xf37\x108D\x7f&!JJ\x836\x83\x9f\n\xf2\x0f\x02!\x00\xd0W\x0ec\xa6\x86C\xf7\x8d\xbe\xf0\xbc2\xe3)|\xbb\xd6\xbb;\xb2\xafG\x05S\x0f-\x14X\n3S\x02 A\xf5\xb20\xcd\xa6<<\x8f\rA\xda\xb68<^\x99\x12x\xb8\xb0\x1b\x9b\xd3%\x8e\xb9\xa6\xf9\xcc\xcf\x83\x02!\x00\xa8.\xee\xdd\xc0\xeb8h\xda\x13^\xb0\xfe\x7fd\xd9]\xa5m\xc66k\xcap`\xe9\xaaav\xe4\xdd#\x02!\x00\xd1\xd0\x93\xe7wZ\xe0p\xd0M\x07\x9e\x13\xa7\xa7\x12\x95\x1e\x13h0O\xc0\x18\x1fa\x10")\x9f&U'

# Valid key in PKCS8 format, invalid cert
try:
    ssl.wrap_socket(io.BytesIO(), key=keypkcs8, cert=b"!")
except ValueError as er:
    print(repr(er))



# test string.splitlines() method

try:
    str.splitlines
except:
    print("SKIP")
    raise SystemExit

# test \n as newline
print("foo\nbar".splitlines())
print("foo\nbar\n".splitlines())
print("foo and\nbar\n".splitlines())
print("foo\nbar\n\n".splitlines())
print("foo\n\nbar\n\n".splitlines())
print("\nfoo\nbar\n".splitlines())

# test \r as newline
print("foo\rbar\r".splitlines())
print("\rfoo and\r\rbar\r".splitlines())

# test \r\n as newline
print("foo\r\nbar\r\n".splitlines())
print("\r\nfoo and\r\n\r\nbar\r\n".splitlines())

# test keepends arg
print("foo\nbar".splitlines(True))
print("foo\nbar\n".splitlines(True))
print("foo\nbar\n\n".splitlines(True))
print("foo\rbar".splitlines(keepends=True))
print("foo\rbar\r\r".splitlines(keepends=True))
print("foo\r\nbar".splitlines(keepends=True))
print("foo\r\nbar\r\n\r\n".splitlines(keepends=True))

# test splitting bytes objects
print(b"foo\nbar".splitlines())
print(b"foo\nbar\n".splitlines())
print(b"foo\r\nbar\r\n\r\n".splitlines(True))



try:
    import errno, os, vfs

    vfs.VfsFat
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class RAMFS_OLD:
    SEC_SIZE = 512

    def __init__(self, blocks):
        self.data = bytearray(blocks * self.SEC_SIZE)

    def readblocks(self, n, buf):
        # print("readblocks(%s, %x(%d))" % (n, id(buf), len(buf)))
        for i in range(len(buf)):
            buf[i] = self.data[n * self.SEC_SIZE + i]

    def writeblocks(self, n, buf):
        # print("writeblocks(%s, %x)" % (n, id(buf)))
        for i in range(len(buf)):
            self.data[n * self.SEC_SIZE + i] = buf[i]

    def sync(self):
        pass

    def count(self):
        return len(self.data) // self.SEC_SIZE


try:
    bdev = RAMFS_OLD(50)
except MemoryError:
    print("SKIP")
    raise SystemExit

vfs.VfsFat.mkfs(bdev)
fs = vfs.VfsFat(bdev)
vfs.mount(fs, "/ramdisk")

# file io
with fs.open("file.txt", "w") as f:
    f.write("hello!")

print(list(fs.ilistdir()))

with fs.open("file.txt", "r") as f:
    print(f.read())

fs.remove("file.txt")
print(list(fs.ilistdir()))



# test subclassing exceptions and providing __new__


class Dummy(BaseException):
    pass


class GoodException(BaseException):
    def __new__(cls, *args, **kwargs):
        print("GoodException __new__")
        return Dummy(*args, **kwargs)


class BadException(BaseException):
    def __new__(cls, *args, **kwargs):
        print("BadException __new__")
        return 1


try:
    raise GoodException("good message")
except BaseException as good:
    print(type(good), good.args[0])

try:
    raise BadException("bad message")
except Exception as bad:
    # Should be TypeError 'exceptions must derive from BaseException'
    print(type(bad), bad.args[0])

try:
    def gen():
        yield

    gen().throw(BadException)
except Exception as genbad:
    # Should be TypeError 'exceptions must derive from BaseException'
    print(type(genbad), genbad.args[0])



try:
    import array
except ImportError:
    print("SKIP")
    raise SystemExit

a = array.array('B', [1, 2, 3])
print(a, len(a))
i = array.array('I', [1, 2, 3])
print(i, len(i))
print(a[0])
print(i[-1])
a = array.array('l', [-1])
print(len(a), a[0])
a1 = array.array('l', [1, 2, 3])
a2 = array.array('L', [1, 2, 3])
print(a2[1])
print(a1 == a2)

# Empty arrays
print(len(array.array('h')))
print(array.array('i'))

# bool operator acting on arrays
print(bool(array.array('i')))
print(bool(array.array('i', [1])))

# containment, with incorrect type
print('12' in array.array('B', b'12'))
print([] in array.array('B', b'12'))

# bad typecode
try:
    array.array('X')
except ValueError:
    print("ValueError")

# equality (CPython requires both sides are array)
print(bytes(array.array('b', [0x61, 0x62, 0x63])) == b'abc')
print(array.array('b', [0x61, 0x62, 0x63]) == b'abc')
print(array.array('B', [0x61, 0x62, 0x63]) == b'abc')
print(array.array('b', [0x61, 0x62, 0x63]) != b'abc')
print(array.array('b', [0x61, 0x62, 0x63]) == b'xyz')
print(array.array('b', [0x61, 0x62, 0x63]) != b'xyz')
print(b'abc' == array.array('b', [0x61, 0x62, 0x63]))
print(b'abc' == array.array('B', [0x61, 0x62, 0x63]))
print(b'abc' != array.array('b', [0x61, 0x62, 0x63]))
print(b'xyz' == array.array('b', [0x61, 0x62, 0x63]))
print(b'xyz' != array.array('b', [0x61, 0x62, 0x63]))

compatible_typecodes = []
for t in ["b", "h", "i", "l", "q"]:
  compatible_typecodes.append((t, t))
  compatible_typecodes.append((t, t.upper()))
for a, b in compatible_typecodes:
  print(array.array(a, [1, 2]) == array.array(b, [1, 2]))

class X(array.array):
    pass

print(bytes(X('b', [0x61, 0x62, 0x63])) == b'abc')
print(X('b', [0x61, 0x62, 0x63]) == b'abc')
print(X('b', [0x61, 0x62, 0x63]) != b'abc')
print(X('b', [0x61, 0x62, 0x63]) == array.array('b', [0x61, 0x62, 0x63]))
print(X('b', [0x61, 0x62, 0x63]) != array.array('b', [0x61, 0x62, 0x63]))

# other comparisons
for typecode in ["B", "H", "I", "L", "Q"]:
    a = array.array(typecode, [1, 1])
    print(a < a)
    print(a <= a)
    print(a > a)
    print(a >= a)

    al = array.array(typecode, [1, 0])
    ab = array.array(typecode, [1, 2])

    print(a < al)
    print(a <= al)
    print(a > al)
    print(a >= al)

    print(a < ab)
    print(a <= ab)
    print(a > ab)
    print(a >= ab)



# test subclassing a native type and overriding __init__

# overriding list.__init__()
class L(list):
    def __init__(self, a, b):
        super().__init__([a, b])
print(L(2, 3))

# with keyword arguments, with star arguments and without because those use different C calls
class D(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
print(D())
print(D([('a', 1)]))
print(D([('a', 1)], a=2, b=3))
print(D(a=2, b=3))

class D(dict):
    def __init__(self):
        super().__init__()
print(D())

class D(dict):
    def __init__(self):
        super().__init__([])
print(D())

class D(dict):
    def __init__(self):
        super().__init__(a=1)
print(D())

class D(dict):
    def __init__(self):
        super().__init__([], a=1)
print(D())

# inherits implicitly from object
class A:
    def __init__(self):
        print("A.__init__")
        super().__init__()
A()

# inherits explicitly from object
class B(object):
    def __init__(self):
        print("B.__init__")
        super().__init__()
B()

# multiple inheritance with object explicitly mentioned
class C:
    pass
class D(C, object):
    def __init__(self):
        print('D.__init__')
        super().__init__()
    def reinit(self):
        print('D.foo')
        super().__init__()
a = D()
a.__init__()
a.reinit()

# call __init__() after object is already init'd
class L(list):
    def reinit(self):
        super().__init__(range(2))
a = L(range(5))
print(a)
a.reinit()
print(a)



# Test generator .throw() with multiple arguments.
# Using multiple arguments is deprecated since CPython 3.12.

# Generator ignores a thrown GeneratorExit (this is allowed).
def gen():
    try:
        yield 123
    except GeneratorExit as e:
        print("GeneratorExit", repr(e.args))
    yield 456


# Thrown an instance with None as second arg.
g = gen()
print(next(g))
print(g.throw(GeneratorExit(), None))

# Thrown a class and instance.
g = gen()
print(next(g))
print(g.throw(GeneratorExit, GeneratorExit(123)))



import io
a = io.StringIO()
print('io.StringIO' in repr(a))
print(a.getvalue())
print(a.read())

a = io.StringIO("foobar")
print(a.getvalue())
print(a.read())
print(a.read())

a = io.StringIO()
a.write("foo")
print(a.getvalue())

a = io.StringIO("foo")
a.write("12")
print(a.getvalue())

a = io.StringIO("foo")
a.write("123")
print(a.getvalue())

a = io.StringIO("foo")
a.write("1234")
print(a.getvalue())

a = io.StringIO()
a.write("foo")
print(a.read())

a = io.StringIO()
print(a.tell())
a.write("foo")
print(a.tell())

a = io.StringIO()
a.close()
for f in [a.read, a.getvalue, lambda:a.write("")]:
    # CPython throws for operations on closed I/O, MicroPython makes
    # the underlying string empty unless MICROPY_CPYTHON_COMPAT defined
    try:
        f()
        print("ValueError")
    except ValueError:
        print("ValueError")



# test native emitter can handle closures correctly


# basic closure
@micropython.native
def f():
    x = 1

    @micropython.native
    def g():
        nonlocal x
        return x

    return g


print(f()())


# closing over an argument
@micropython.native
def f(x):
    @micropython.native
    def g():
        nonlocal x
        return x

    return g


print(f(2)())


# closing over an argument and a normal local
@micropython.native
def f(x):
    y = 2 * x

    @micropython.native
    def g(z):
        return x + y + z

    return g


print(f(2)(3))



"""
os module test for the CC3200 based boards
"""

from machine import SD
import os, vfs

mch = os.uname().machine
if "LaunchPad" in mch:
    sd_pins = ("GP16", "GP17", "GP15")
elif "WiPy" in mch:
    sd_pins = ("GP10", "GP11", "GP15")
else:
    raise Exception("Board not supported!")

sd = SD(pins=sd_pins)

vfs.mount(sd, "/sd")
os.mkfs("/sd")
os.chdir("/flash")
print(os.listdir())

os.chdir("/sd")
print(os.listdir())

# create a test directory in flash
os.mkdir("/flash/test")
os.chdir("/flash/test")
print(os.getcwd())
os.chdir("..")
print(os.getcwd())
os.chdir("test")
print(os.getcwd())
# create a new file
f = open("test.txt", "w")
test_bytes = os.urandom(1024)
n_w = f.write(test_bytes)
print(n_w == len(test_bytes))
f.close()
f = open("test.txt", "r")
r = bytes(f.read(), "ascii")
# check that we can write and read it correctly
print(r == test_bytes)
f.close()
os.rename("test.txt", "newtest.txt")
print(os.listdir())
os.rename("/flash/test", "/flash/newtest")
print(os.listdir("/flash"))
os.remove("newtest.txt")
os.chdir("..")
os.rmdir("newtest")

# create a test directory in the sd card
os.mkdir("/sd/test")
os.chdir("/sd/test")
print(os.getcwd())
os.chdir("..")
print(os.getcwd())
os.chdir("test")
print(os.getcwd())
# create a new file
f = open("test.txt", "w")
test_bytes = os.urandom(1024)
n_w = f.write(test_bytes)
print(n_w == len(test_bytes))
f.close()
f = open("test.txt", "r")
r = bytes(f.read(), "ascii")
# check that we can write and read it correctly
print(r == test_bytes)
f.close()

print("CC3200" in os.uname().machine)
print("WiPy" == os.uname().sysname)

os.sync()
os.stat("/flash")
os.stat("/flash/sys")
os.stat("/flash/boot.py")
os.stat("/sd")
os.stat("/")
os.chdir("/sd/test")
os.remove("test.txt")
os.chdir("/sd")
os.rmdir("test")
os.listdir("/sd")
print(os.listdir("/"))
os.unmount("/sd")
print(os.listdir("/"))
os.mkfs(sd)
vfs.mount(sd, "/sd")
print(os.listdir("/"))
os.chdir("/flash")

# next ones must raise
sd.deinit()
try:
    os.listdir("/sd")
except:
    print("Exception")

# re-initialization must work
sd.init()
print(os.listdir("/sd"))

try:
    vfs.mount(sd, "/sd")
except:
    print("Exception")

try:
    vfs.mount(sd, "/sd2")
except:
    print("Exception")

os.unmount("/sd")
try:
    os.listdir("/sd")
except:
    print("Exception")

try:
    os.unmount("/flash")
except:
    print("Exception")

try:
    os.unmount("/something")
except:
    print("Exception")

try:
    os.unmount("something")
except:
    print("Exception")

try:
    os.mkfs("flash")  # incorrect path format
except:
    print("Exception")

try:
    os.remove("/flash/nofile.txt")
except:
    print("Exception")

try:
    os.rename("/flash/nofile.txt", "/flash/nofile2.txt")
except:
    print("Exception")

try:
    os.chdir("/flash/nodir")
except:
    print("Exception")

try:
    os.listdir("/flash/nodir")
except:
    print("Exception")

vfs.mount(sd, "/sd")
print(os.listdir("/"))
os.unmount("/sd")



import time
import machine
from hwconfig import LED, BUTTON

# machine.time_pulse_us() function demo

print(
    """\
Let's play an interesting game:
You click button as fast as you can, and I tell you how slow you are.
Ready? Cliiiiick!
"""
)

while 1:
    delay = machine.time_pulse_us(BUTTON, 1, 10 * 1000 * 1000)
    if delay < 0:
        print("Well, you're *really* slow")
    else:
        print("You are as slow as %d microseconds!" % delay)
    time.sleep_ms(10)



# Test advertising multiple services, and scanning them.

import sys

# ruff: noqa: E402
sys.path.append("")

import asyncio
import aioble
import bluetooth

TIMEOUT_MS = 5000

_SERVICE_16_A = bluetooth.UUID(0x180F)  # Battery Service
_SERVICE_16_B = bluetooth.UUID(0x181A)  # Environmental Sensing Service
_SERVICE_32_A = bluetooth.UUID("AB12")  # random
_SERVICE_32_B = bluetooth.UUID("CD34")  # random


# Acting in peripheral role (advertising).
async def instance0_task():
    multitest.globals(BDADDR=aioble.config("mac"))
    multitest.next()

    # Advertise, and wait for central to connect to us.
    print("advertise")
    async with await aioble.advertise(
        20_000,
        name="MPY",
        services=[_SERVICE_16_A, _SERVICE_16_B, _SERVICE_32_A, _SERVICE_32_B],
        timeout_ms=TIMEOUT_MS,
    ) as connection:
        print("connected")
        await connection.disconnected()
        print("disconnected")


def instance0():
    try:
        asyncio.run(instance0_task())
    finally:
        aioble.stop()


# Acting in central role (scanning).
async def instance1_task():
    multitest.next()

    wanted_device = aioble.Device(*BDADDR)

    # Scan for the wanted device/peripheral and print its advertised services.
    async with aioble.scan(5000, interval_us=30000, window_us=30000, active=True) as scanner:
        async for result in scanner:
            if result.device == wanted_device:
                services = list(result.services())
                if services:
                    print(services)
                    break

    # Connect to peripheral and then disconnect.
    print("connect")
    device = aioble.Device(*BDADDR)
    async with await device.connect(timeout_ms=TIMEOUT_MS):
        print("disconnect")


def instance1():
    try:
        asyncio.run(instance1_task())
    finally:
        aioble.stop()



# Test that native text/BSS/rodata loaded from a .mpy file is retained after a GC.

try:
    import gc, sys, io, vfs

    sys.implementation._mpy
    io.IOBase
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class UserFile(io.IOBase):
    def __init__(self, data):
        self.data = memoryview(data)
        self.pos = 0

    def readinto(self, buf):
        n = min(len(buf), len(self.data) - self.pos)
        buf[:n] = self.data[self.pos : self.pos + n]
        self.pos += n
        return n

    def ioctl(self, req, arg):
        return 0


class UserFS:
    def __init__(self, files):
        self.files = files

    def mount(self, readonly, mksfs):
        pass

    def umount(self):
        pass

    def stat(self, path):
        if path in self.files:
            return (32768, 0, 0, 0, 0, 0, 0, 0, 0, 0)
        raise OSError

    def open(self, path, mode):
        return UserFile(self.files[path])


# Pre-compiled import_mpy_native_gc_module example for various architectures, keyed
# by the required value of sys.implementation._mpy (without sub-version).
# To rebuild:
#   $ cd import_mpy_native_gc_module
#   $ make clean
#   $ make ARCH=x64 # or ARCH=armv6m or ARCH=xtensawin
# Then copy the bytes object printed on the last line.
features0_file_contents = {
    # -march=x64
    0x806: b"M\x06\x0b\x1f\x03\x002build/test_x64.native.mpy\x00\x08add1\x00\x0cunused\x00\x91B\xe9I\x00\x00\x00H\x8b\x05\xf4\x00\x00\x00H\x8b\x00\xc3H\x8b\x05\xf9\x00\x00\x00\xbe\x02\x00\x00\x00\x8b8H\x8b\x05\xdb\x00\x00\x00H\x8b@ \xff\xe0H\x8b\x05\xce\x00\x00\x00S\xbe\x02\x00\x00\x00H\x8bX \xffP\x18\xbe\x02\x00\x00\x00H\x8dx\x01H\x89\xd8[\xff\xe0AVAUATUSH\x8b\x1d\xa3\x00\x00\x00H\x8bG\x08L\x8bk(H\x8bx\x08A\xff\xd5L\x8b5\x95\x00\x00\x00L\x8bchH\x8d5r\x00\x00\x00H\x89\xc5H\x8b\x05\x88\x00\x00\x00A\x0f\xb7~\x04\xc7\x00@\xe2\x01\x00A\xff\xd4H\x8d5C\x00\x00\x00\xbfV\x00\x00\x00A\xff\xd4A\x0f\xb7~\x02H\x8d5\x1f\x00\x00\x00A\xff\xd4H\x89\xefA\xff\xd5H\x8b\x03[]A\\A]A^\xc3\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00+\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00P\x04\x11@\rB\tD\xaf4\x016\xad8\x01:\xaf<\x01>\xff",
    # -march=armv6m
    0x1006: b"M\x06\x13\x1f\x03\x008build/test_armv6m.native.mpy\x00\x08add1\x00\x0cunused\x00\x8eb0\xe0\x00\x00\x00\x00\x00\x00\x02K\x03J{D\x9bX\x18hpG\xd0\x00\x00\x00\x00\x00\x00\x00\x10\xb5\x05K\x05I\x06J{D\x9aX[X\x10h\x02!\x1bi\x98G\x10\xbd\xb8\x00\x00\x00\x00\x00\x00\x00\x08\x00\x00\x00\x10\xb5\x06K\x06J{D\x9bX\x02!\x1ci\xdbh\x98G\x02!\x010\xa0G\x10\xbd\xc0F\x96\x00\x00\x00\x00\x00\x00\x00\xf7\xb5\x12O\x12K\x7fD\xfdX\x12Lki|D\x00\x93ChXh\x00\x9b\x98G\x0fK\x01\x90\x0fJ\xfbXnk\x1a`\x0eK!\x00\xffX\xb8\x88\xb0G!\x00V \x081\xb0G!\x00x\x88\x101\xb0G\x01\x98\x00\x9b\x98G(h\xfe\xbd\xc0Fr\x00\x00\x00\x00\x00\x00\x00R\x00\x00\x00\x08\x00\x00\x00@\xe2\x01\x00\x04\x00\x00\x00\x00\x00\x00\x00\t\x00\x00\x00\x00\x00\x00\x00\x1d\x00\x00\x00\x00\x00\x00\x00A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00P\x04\x11p\rr\tt\xafd\x01f\xadh\x01j\xafl\x01n\xff",
    # -march=xtensawin
    0x2806: b"M\x06+\x1f\x03\x00>build/test_xtensawin.native.mpy\x00\x08add1\x00\x0cunused\x00\x8a\x12\x06\x16\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x006A\x00\x81\xf9\xff(\x08\x1d\xf0\x00\x006A\x00\x91\xfb\xff\x81\xf5\xff\xa8\t\x88H\x0c+\xe0\x08\x00-\n\x1d\xf0\x00\x006A\x00\x81\xf0\xff\xad\x02xH\x888\x0c+\xe0\x08\x00\x0c+\x1b\xaa\xe0\x07\x00-\n\x1d\xf06A\x00a\xe9\xff\x88\x122&\x05\xa2(\x01\xe0\x03\x00q\xe6\xff\x81\xea\xff\x92\xa7\x89\xa0\x99\x11H\xd6]\n\xb1\xe3\xff\xa2\x17\x02\x99\x08\xe0\x04\x00\xb1\xe2\xff\\j\xe0\x04\x00\xb1\xe1\xff\xa2\x17\x01\xe0\x04\x00\xad\x05\xe0\x03\x00(\x06\x1d\xf0p\x18\x04\x00\x00\x00\x00@\x00\x00\x00\x00\x00\x00\x00(\x00\x00\x00\x00\x00\x00\x00\x1c\x00\x00\x00\x11\x02\r\x04\x07\x06\x03\t\x0c\xaf\x01\x01\x03\xad\x05\x01\x07\xaf\t\x01\x0b\xff",
}

# Populate armv7m-derived archs based on armv6m.
for arch in (0x1406, 0x1806, 0x1C06, 0x2006):
    features0_file_contents[arch] = features0_file_contents[0x1006]

# Check that a .mpy exists for the target (ignore sub-version in lookup).
sys_implementation_mpy = sys.implementation._mpy & ~(3 << 8)
if sys_implementation_mpy not in features0_file_contents:
    print("SKIP")
    raise SystemExit

# These are the test .mpy files.
user_files = {"/features0.mpy": features0_file_contents[sys_implementation_mpy]}

# Create and mount a user filesystem.
vfs.mount(UserFS(user_files), "/userfs")
sys.path.append("/userfs")

# Import the native function.
gc.collect()
from features0 import get, add1

# Test that the native functions work to begin with.
print(get())
print(add1(12))

# Free the module that contained the function.
del sys.modules["features0"]


# Sweep the stack to remove any stray pointers that we are aiming to reclaim.
def recurse(n):
    if n:
        recurse(n - 1)


recurse(10)

# Run a GC cycle which should reclaim the module but not the function.
gc.collect()

# Allocate lots of fragmented memory to overwrite anything that was just freed by the GC.
for i in range(1000):
    []

# Run the native function, its text/BSS/rodata should not have been freed or overwritten.
print(get())
print(add1(12))

# Unmount and undo path addition.
vfs.umount("/userfs")
sys.path.pop()



# Test ESP32 OTA updates, including automatic roll-back.
# Running this test requires firmware with an OTA Partition, such as the GENERIC_OTA "board".
# This test also requires patience as it copies the boot partition into the other OTA slot.

import machine
from esp32 import Partition

# start by checking that the running partition table has OTA partitions, 'cause if
# it doesn't there's nothing we can test
cur = Partition(Partition.RUNNING)
cur_name = cur.info()[4]
if not cur_name.startswith("ota_"):
    print("SKIP")
    raise SystemExit

DEBUG = True


def log(*args):
    if DEBUG:
        print(*args)


# replace boot.py with the test code that will run on each reboot
import os

try:
    os.rename("boot.py", "boot-orig.py")
except:
    pass
with open("boot.py", "w") as f:
    f.write("DEBUG=" + str(DEBUG))
    f.write(
        """
import machine
from esp32 import Partition
cur = Partition(Partition.RUNNING)
cur_name = cur.info()[4]

def log(*args):
    if DEBUG: print(*args)

from step import STEP, EXPECT
log("Running partition: " + cur_name + " STEP=" + str(STEP) + " EXPECT=" + EXPECT)
if cur_name != EXPECT:
    print("\\x04FAILED: step " + str(STEP) + " expected " + EXPECT + " got " + cur_name + "\\x04")

if STEP == 0:
    log("Not confirming boot ok and resetting back into first")
    nxt = cur.get_next_update()
    with open("step.py", "w") as f:
        f.write("STEP=1\\nEXPECT=\\"" + nxt.info()[4] + "\\"\\n")
    machine.reset()
elif STEP == 1:
    log("Booting into second partition again")
    nxt = cur.get_next_update()
    nxt.set_boot()
    with open("step.py", "w") as f:
        f.write("STEP=2\\nEXPECT=\\"" + nxt.info()[4] + "\\"\\n")
    machine.reset()
elif STEP == 2:
    log("Confirming boot ok and rebooting into same partition")
    Partition.mark_app_valid_cancel_rollback()
    with open("step.py", "w") as f:
        f.write("STEP=3\\nEXPECT=\\"" + cur_name + "\\"\\n")
    machine.reset()
elif STEP == 3:
    log("Booting into original partition")
    nxt = cur.get_next_update()
    nxt.set_boot()
    with open("step.py", "w") as f:
        f.write("STEP=4\\nEXPECT=\\"" + nxt.info()[4] + "\\"\\n")
    machine.reset()
elif STEP == 4:
    log("Confirming boot ok and DONE!")
    Partition.mark_app_valid_cancel_rollback()
    import os
    os.remove("step.py")
    os.remove("boot.py")
    os.rename("boot-orig.py", "boot.py")
    print("\\nSUCCESS!\\n\\x04\\x04")

"""
    )


def copy_partition(src, dest):
    log("Partition copy: {} --> {}".format(src.info(), dest.info()))
    sz = src.info()[3]
    if dest.info()[3] != sz:
        raise ValueError("Sizes don't match: {} vs {}".format(sz, dest.info()[3]))
    addr = 0
    blk = bytearray(4096)
    while addr < sz:
        if sz - addr < 4096:
            blk = blk[: sz - addr]
        if addr & 0xFFFF == 0:
            # need to show progress to run-tests.py else it times out
            print("   ... 0x{:06x}".format(addr))
        src.readblocks(addr >> 12, blk)
        dest.writeblocks(addr >> 12, blk)
        addr += len(blk)


# get things started by copying the current partition into the next slot and rebooting
print("Copying current to next partition")
nxt = cur.get_next_update()
copy_partition(cur, nxt)
print("Partition copied, booting into it")
nxt.set_boot()

# the step.py file is used to keep track of state across reboots
# EXPECT is the name of the partition we expect to reboot into
with open("step.py", "w") as f:
    f.write('STEP=0\nEXPECT="' + nxt.info()[4] + '"\n')

machine.reset()



import os, pyb

machine = os.uname().machine
if "PYBv1." in machine or "PYBLITEv1." in machine:
    leds = [pyb.LED(i) for i in range(1, 5)]
    pwm_leds = leds[2:]
elif "PYBD" in machine:
    leds = [pyb.LED(i) for i in range(1, 4)]
    pwm_leds = []
else:
    print("SKIP")
    raise SystemExit

# test printing
for i in range(3):
    print(leds[i])

# test on and off
for l in leds:
    l.on()
    assert l.intensity() == 255
    pyb.delay(100)
    l.off()
    assert l.intensity() == 0
    pyb.delay(100)

# test toggle
for l in 2 * leds:
    l.toggle()
    assert l.intensity() in (0, 255)
    pyb.delay(100)

# test intensity
for l in pwm_leds:
    for i in range(256):
        l.intensity(i)
        assert l.intensity() == i
        pyb.delay(1)
    for i in range(255, -1, -1):
        l.intensity(i)
        assert l.intensity() == i
        pyb.delay(1)



#!/usr/bin/env python3
#
# Host side of the `usb_simple_device.py` example.  This must be run using standard
# Python on a PC.  See further instructions in `usb_simple_device.py`.

import sys
import usb.core
import usb.util

# VID and PID of the custom USB device.
VID = 0xF055
PID = 0x9999

# USB endpoints used by the device.
EP_OUT = 0x01
EP_IN = 0x81


def main():
    # Search for the custom USB device by VID/PID.
    dev = usb.core.find(idVendor=VID, idProduct=PID)

    if dev is None:
        print("No USB device found")
        sys.exit(1)

    # Claim the USB device.
    usb.util.claim_interface(dev, 0)

    # Read the device's strings.
    for i in range(0x11, 0x17):
        print(f"str{i}:", usb.util.get_string(dev, i))

    # Test writing to the device.
    ret = dev.write(EP_OUT, b"01234567", timeout=1000)
    print(ret)

    # Test reading from the device.
    print(dev.read(EP_IN, 64))

    # Release the USB device.
    usb.util.release_interface(dev, 0)
    usb.util.dispose_resources(dev)


if __name__ == "__main__":
    main()



# tests logical constant folding in parser

def f_true():
    print('f_true')
    return True

def f_false():
    print('f_false')
    return False

print(0 or False)
print(1 or foo)
print(f_false() or 1 or foo)
print(f_false() or 1 or f_true())

print(0 and foo)
print(1 and True)
print(f_true() and 0 and foo)
print(f_true() and 1 and f_false())

print(not 0)
print(not False)
print(not 1)
print(not True)
print(not not 0)
print(not not 1)



try:
    # Check if deflate is available.
    import deflate
    import io
except ImportError:
    print("SKIP")
    raise SystemExit

try:
    # Check there's enough memory to deflate gzip streams.
    # zlib.compress(b'', wbits=25)
    empty_gzip = (
        b"\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00"
    )
    deflate.DeflateIO(io.BytesIO(empty_gzip)).read()
except MemoryError:
    print("SKIP")
    raise SystemExit

# zlib.compress(b'micropython hello world hello world micropython', wbits=-9)
data_raw = b'\xcb\xcdL.\xca/\xa8,\xc9\xc8\xcfS\xc8H\xcd\xc9\xc9W(\xcf/\xcaIAa\xe7"\xd4\x00\x00'
# zlib.compress(b'micropython hello world hello world micropython', wbits=9)
data_zlib = b'\x18\x95\xcb\xcdL.\xca/\xa8,\xc9\xc8\xcfS\xc8H\xcd\xc9\xc9W(\xcf/\xcaIAa\xe7"\xd4\x00\x00\xbc\xfa\x12\x91'
# zlib.compress(b'micropython hello world hello world micropython', wbits=25)
data_gzip = b'\x1f\x8b\x08\x00\x00\x00\x00\x00\x00\x03\xcb\xcdL.\xca/\xa8,\xc9\xc8\xcfS\xc8H\xcd\xc9\xc9W(\xcf/\xcaIAa\xe7"\xd4\x00\x00"\xeb\xc4\x98/\x00\x00\x00'

# compress(b'hello' + bytearray(300) + b'hello', format=deflate.RAW, 5)
data_wbits_5 = b"\xcbH\xcd\xc9\xc9g\x18\xe9\x00\x08\x88\x95\xcfH\xcd\xc9\xc9\x07\x00"
# compress(b'hello' + bytearray(300) + b'hello', format=deflate.RAW, 6)
data_wbits_6 = b"\xcbH\xcd\xc9\xc9g\x18\xe9\x00\x08\x88\xd5\x9f\x91\x9a\x93\x93\x0f\x00"
# compress(b'hello' + bytearray(300) + b'hello', format=deflate.RAW, 8)
data_wbits_8 = b"\xcbH\xcd\xc9\xc9g\x18\xe9\x00\x08\x88\xf5\x7fFjNN>\x00"
# compress(b'hello' + bytearray(2000) + b'hello', format=deflate.RAW, 10)
data_wbits_10 = b"\xcbH\xcd\xc9\xc9g\x18\xe9\x00\x08Fz\x18\x00\xc3`\xa4'\x03`2\x18\xe99\x01\x98\x13Fz\xfe\x07\xe6\xff\x91\x9e\xff\x81\xf9\x7f\xa4\xe7\x7f`\xfe\x1f\xba\xf9?#5''\x1f\x00"


def decompress(data, *args):
    buf = io.BytesIO(data)
    with deflate.DeflateIO(buf, *args) as g:
        return g.read()


def decompress_error(data, *args):
    try:
        decompress(data, *args)
    except OSError:
        print("OSError")
    except EOFError:
        print("EOFError")
    except ValueError:
        print("ValueError")


# Basic handling of format and detection.
print(decompress(data_raw, deflate.RAW))
print(decompress(data_zlib, deflate.ZLIB))
print(decompress(data_gzip, deflate.GZIP))
print(decompress(data_zlib))  # detect zlib/gzip.
print(decompress(data_gzip))  # detect zlib/gzip.

decompress_error(data_raw)  # cannot detect zlib/gzip from raw stream
decompress_error(data_raw, deflate.ZLIB)
decompress_error(data_raw, deflate.GZIP)
decompress_error(data_zlib, deflate.RAW)
decompress_error(data_zlib, deflate.GZIP)
decompress_error(data_gzip, deflate.RAW)
decompress_error(data_gzip, deflate.ZLIB)

# Invalid data stream.
decompress_error(b"abcef", deflate.RAW)

# Invalid block type. final-block, block-type=3.
decompress_error(b"\x07", deflate.RAW)

# Truncated stream.
decompress_error(data_raw[:10], deflate.RAW)

# Partial reads.
buf = io.BytesIO(data_zlib)
with deflate.DeflateIO(buf) as g:
    print(buf.seek(0, 1))  # verify stream is not read until first read of the DeflateIO stream.
    print(g.read(1))
    print(buf.seek(0, 1))  # verify that only the minimal amount is read from the source
    print(g.read(1))
    print(buf.seek(0, 1))
    print(g.read(2))
    print(buf.seek(0, 1))
    print(g.read())
    print(buf.seek(0, 1))
    print(g.read(1))
    print(buf.seek(0, 1))
    print(g.read())

# Invalid zlib checksum (+ length for gzip). Note: only checksum errors are
# currently detected, see the end of uzlib_uncompress_chksum().
decompress_error(data_zlib[:-4] + b"\x00\x00\x00\x00")
decompress_error(data_gzip[:-8] + b"\x00\x00\x00\x00\x00\x00\x00\x00")
decompress_error(data_zlib[:-4] + b"\x00\x00\x00\x00", deflate.ZLIB)
decompress_error(data_gzip[:-8] + b"\x00\x00\x00\x00\x00\x00\x00\x00", deflate.GZIP)

# Reading from a closed underlying stream.
b = io.BytesIO(data_raw)
g = deflate.DeflateIO(b, deflate.RAW)
g.read(4)
b.close()
try:
    g.read(4)
except ValueError:
    print("ValueError")

# Reading from a closed DeflateIO.
b = io.BytesIO(data_raw)
g = deflate.DeflateIO(b, deflate.RAW)
g.read(4)
g.close()
try:
    g.read(4)
except OSError:
    print("OSError")

# Gzip header with extra flags (FCOMMENT FNAME FEXTRA FHCRC) enabled.
data_gzip_header_extra = b"\x1f\x8b\x08\x1e}\x9a\x9bd\x02\x00\x00\x00\x00\x00\x00\xff\xcb\xcdL.\xca/\xa8,\xc9\xc8\xcf\x03\x00\xf2KF>\x0b\x00\x00\x00"
print(decompress(data_gzip_header_extra))

# Test patterns.
PATTERNS_ZLIB = [
    # Packed results produced by CPy's zlib.compress()
    (b"0", b"x\x9c3\x00\x00\x001\x001"),
    (b"a", b"x\x9cK\x04\x00\x00b\x00b"),
    (b"0" * 100, b"x\x9c30\xa0=\x00\x00\xb3q\x12\xc1"),
    (
        bytes(range(64)),
        b"x\x9cc`dbfaec\xe7\xe0\xe4\xe2\xe6\xe1\xe5\xe3\x17\x10\x14\x12\x16\x11\x15\x13\x97\x90\x94\x92\x96\x91\x95\x93WPTRVQUS\xd7\xd0\xd4\xd2\xd6\xd1\xd5\xd370426153\xb7\xb0\xb4\xb2\xb6\xb1\xb5\xb3\x07\x00\xaa\xe0\x07\xe1",
    ),
    (b"hello", b"x\x01\x01\x05\x00\xfa\xffhello\x06,\x02\x15"),  # compression level 0
    # adaptive/dynamic huffman tree
    (
        b"13371813150|13764518736|12345678901",
        b"x\x9c\x05\xc1\x81\x01\x000\x04\x04\xb1\x95\\\x1f\xcfn\x86o\x82d\x06Qq\xc8\x9d\xc5X}<e\xb5g\x83\x0f\x89X\x07\xab",
    ),
    # dynamic Huffman tree with "case 17" (repeat code for 3-10 times)
    (
        b">I}\x00\x951D>I}\x00\x951D>I}\x00\x951D>I}\x00\x951D",
        b"x\x9c\x05\xc11\x01\x00\x00\x00\x010\x95\x14py\x84\x12C_\x9bR\x8cV\x8a\xd1J1Z)F\x1fw`\x089",
    ),
]
for unpacked, packed in PATTERNS_ZLIB:
    print(decompress(packed) == unpacked)
    print(decompress(packed, deflate.ZLIB) == unpacked)

# Older version's of CPython's zlib module still included the checksum and length (as if it were a zlib/gzip stream).
# Make sure there're no problem decompressing this.
data_raw_with_footer = data_raw + b"\x00\x00\x00\x00\x00\x00\x00\x00"
print(decompress(data_raw_with_footer, deflate.RAW))

# Valid wbits values.
decompress_error(data_wbits_5, deflate.RAW, -1)
print(len(decompress(data_wbits_5, deflate.RAW, 0)))
decompress_error(data_wbits_5, deflate.RAW, 1)
decompress_error(data_wbits_5, deflate.RAW, 4)
for i in range(5, 16):
    print(len(decompress(data_wbits_5, deflate.RAW, i)))
decompress_error(data_wbits_5, deflate.RAW, 16)

# Invalid values for format.
decompress_error(data_raw, -1)
decompress_error(data_raw, 5)

# Data that requires a higher wbits value.
decompress_error(data_wbits_6, deflate.RAW, 5)
print(len(decompress(data_wbits_6, deflate.RAW, 6)))
print(len(decompress(data_wbits_6, deflate.RAW, 7)))
decompress_error(data_wbits_8, deflate.RAW, 7)
print(len(decompress(data_wbits_8, deflate.RAW, 8)))
print(len(decompress(data_wbits_8, deflate.RAW, 9)))
decompress_error(data_wbits_10, deflate.RAW)
decompress_error(data_wbits_10, deflate.RAW, 9)
print(len(decompress(data_wbits_10, deflate.RAW, 10)))

# zlib header sets the size, so works with wbits unset or wbits >= 10.
data_wbits_10_zlib = b"(\x91\xcbH\xcd\xc9\xc9g\x18\xe9\x00\x08Fz\x18\x00\xc3`\xa4'\x03`2\x18\xe99\x01\x98\x13Fz\xfe\x07\xe6\xff\x91\x9e\xff\x81\xf9\x7f\xa4\xe7\x7f`\xfe\x1f\xba\xf9?#5''\x1f\x00[\xbc\x04)"
print(len(decompress(data_wbits_10_zlib, deflate.ZLIB)))
decompress_error(data_wbits_10_zlib, deflate.ZLIB, 9)
print(len(decompress(data_wbits_10_zlib, deflate.ZLIB, 10)))
print(len(decompress(data_wbits_10_zlib)))



"""
The MIT License (MIT)

Copyright (c) 2023 Arduino SA

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

HS3003 driver for MicroPython.

Example usage:

import time
from hs3003 import HS3003
from machine import Pin, I2C

bus = I2C(1, scl=Pin(15), sda=Pin(14))
hts = HS3003(bus)

while True:
    rH   = hts.humidity()
    temp = hts.temperature()
    print ("rH: %.2f%% T: %.2fC" %(rH, temp))
    time.sleep_ms(100)
"""

import struct


class HS3003:
    def __init__(self, bus, address=0x44):
        self.bus = bus
        self.address = address

    def _read_data(self):
        # Init measurement mode
        self.bus.writeto(self.address, b"")
        # Data fetch
        return struct.unpack(">HH", self.bus.readfrom(self.address, 4))

    def humidity(self):
        """Returns the relative humidity in percent."""
        h, t = self._read_data()
        return ((h & 0x3FFF) / 16383) * 100

    def temperature(self):
        """Returns the temperature in degrees Celsius."""
        h, t = self._read_data()
        return ((t >> 2) / 16383) * 165 - 40



# test calling a function with keywords given by **dict

def f(a, b):
    print(a, b)

f(1, **{'b':2})
f(1, **{'b':val for val in range(1)})

try:
    f(1, **{len:2})
except TypeError:
    print('TypeError')

# test calling a method with keywords given by **dict

class A:
    def f(self, a, b):
        print(a, b)

a = A()
a.f(1, **{'b':2})
a.f(1, **{'b':val for val in range(1)})

# test for duplicate keys in **kwargs not allowed

def f1(**kwargs):
    print(kwargs)

try:
    f1(**{'a': 1, 'b': 2}, **{'b': 3, 'c': 4})
except TypeError:
    print('TypeError')



import io
try:
    io.IOBase
except AttributeError:
    print('SKIP')
    raise SystemExit


class MyIO(io.IOBase):
    def write(self, buf):
        # CPython and uPy pass in different types for buf (str vs bytearray)
        print('write', len(buf))
        return len(buf)

print('test', file=MyIO())



# test importing of .mpy files with native code

try:
    import sys, io, vfs

    sys.implementation._mpy
    io.IOBase
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

mpy_arch = sys.implementation._mpy >> 8
if mpy_arch >> 2 == 0:
    # This system does not support .mpy files containing native code
    print("SKIP")
    raise SystemExit


class UserFile(io.IOBase):
    def __init__(self, data):
        self.data = memoryview(data)
        self.pos = 0

    def readinto(self, buf):
        n = min(len(buf), len(self.data) - self.pos)
        buf[:n] = self.data[self.pos : self.pos + n]
        self.pos += n
        return n

    def ioctl(self, req, arg):
        return 0


class UserFS:
    def __init__(self, files):
        self.files = files

    def mount(self, readonly, mksfs):
        pass

    def umount(self):
        pass

    def stat(self, path):
        if path in self.files:
            return (32768, 0, 0, 0, 0, 0, 0, 0, 0, 0)
        raise OSError

    def open(self, path, mode):
        return UserFile(self.files[path])


# these are the test .mpy files
valid_header = bytes([77, 6, mpy_arch, 31])
# fmt: off
user_files = {
    # bad architecture (mpy_arch needed for sub-version)
    '/mod0.mpy': bytes([77, 6, 0xfc | mpy_arch, 31]),

    # test loading of viper and asm
    '/mod1.mpy': valid_header + (
        b'\x02' # n_qstr
        b'\x00' # n_obj

        b'\x0emod1.py\x00' # qstr0 = "mod1.py"
        b'\x0aouter\x00' # qstr1 = "outer"

        b'\x2c' # 5 bytes, have children, bytecode
            b'\x00\x02' # prelude
            b'\x01' # simple name (qstr index)
            b'\x51' # LOAD_CONST_NONE
            b'\x63' # RETURN_VALUE

        b'\x02' # 2 children

            b'\x42' # 8 bytes, no children, viper code
                b'\x00\x00\x00\x00\x00\x00\x00\x00' # dummy machine code
                b'\x00' # scope_flags

            b'\x43' # 8 bytes, no children, asm code
                b'\x00\x00\x00\x00\x00\x00\x00\x00' # dummy machine code
                b'\x00\x00\x00' # scope_flags, n_pos_args, type_sig
    ),

    # test loading viper with additional scope flags and relocation
    '/mod2.mpy': valid_header + (
        b'\x02' # n_qstr
        b'\x00' # n_obj

        b'\x0emod2.py\x00' # qstr0 = "mod2.py"
        b'\x0aouter\x00' # qstr1 = "outer"

        b'\x2c' # 5 bytes, have children, bytecode
            b'\x00\x02' # prelude
            b'\x01' # simple name (qstr index)
            b'\x51' # LOAD_CONST_NONE
            b'\x63' # RETURN_VALUE

        b'\x01' # 1 child

            b'\x22' # 4 bytes, no children, viper code
                b'\x00\x00\x00\x00' # dummy machine code
                b'\x70' # scope_flags: VIPERBSS | VIPERRODATA | VIPERRELOC
                b'\x06\x04' # rodata=6 bytes, bss=4 bytes
                b'rodata' # rodata content
                b'\x03\x01\x00' # dummy relocation of rodata
    ),
}
# fmt: on

# create and mount a user filesystem
vfs.mount(UserFS(user_files), "/userfs")
sys.path.append("/userfs")

# import .mpy files from the user filesystem
for i in range(len(user_files)):
    mod = "mod%u" % i
    try:
        __import__(mod)
        print(mod, "OK")
    except ValueError as er:
        print(mod, "ValueError", er)

# unmount and undo path addition
vfs.umount("/userfs")
sys.path.pop()



# test __dict__ attribute of a user module

import sys

if not hasattr(sys, "__dict__"):
    print("SKIP")
    raise SystemExit


import import1b

# dict of a user module (read/write)
print(import1b.var)
print(import1b.__dict__["var"])
import1b.__dict__["var"] = "hello"
print(import1b.var)
print(import1b.__dict__["var"])



import re

m = re.search(r"a+", "caaab")
assert m.group(0) == "aaa"
assert m.group() == "aaa"

m = re.match(r"(?ms)foo.*\Z", "foo\nbar")
assert m.group(0) == "foo\nbar"

assert re.match(r"a+", "caaab") is None
m = re.match(r"a+", "aaaab")
assert m.group(0) == "aaaa"

assert re.sub("a", "z", "caaab") == "czzzb"
assert re.sub("a+", "z", "caaab") == "czb"
assert re.sub("a", "z", "caaab", 1) == "czaab"
assert re.sub("a", "z", "caaab", 2) == "czzab"
assert re.sub("a", "z", "caaab", 10) == "czzzb"

assert re.sub("a", lambda m: m.group(0) * 2, "caaab") == "caaaaaab"

m = re.match(r"(\d+)\.(\d+)", "24.1632")
assert m.groups() == ("24", "1632")
assert m.group(2, 1) == ("1632", "24")

assert re.escape(r"1243*&[]_dsfAd") == r"1243\*\&\[\]_dsfAd"

assert re.split("x*", "foo") == ["foo"]
assert re.split("(?m)^$", "foo\n\nbar\n") == ["foo\n\nbar\n"]
assert re.split("\W+", "Words, words, words.") == ["Words", "words", "words", ""]
assert re.split("(\W+)", "Words, words, words.") == [
    "Words",
    ", ",
    "words",
    ", ",
    "words",
    ".",
    "",
]
assert re.split("\W+", "Words, words, words.", 1) == ["Words", "words, words."]
assert re.split("[a-f]+", "0a3B9", flags=re.IGNORECASE) == ["0", "3", "9"]
assert re.split("(\W+)", "...words, words...") == ["", "...", "words", ", ", "words", "...", ""]

assert re.sub(r"[ :/?&]", "_", "http://foo.ua/bar/?a=1&b=baz/") == "http___foo.ua_bar__a=1_b=baz_"

text = "He was carefully disguised but captured quickly by police."
assert re.findall(r"\w+ly", text) == ["carefully", "quickly"]

text = "He was carefully disguised but captured quickly by police."
assert re.findall(r"(\w+)(ly)", text) == [("careful", "ly"), ("quick", "ly")]

text = "He was carefully disguised but captured quickly by police."
assert re.findall(r"(\w+)ly", text) == ["careful", "quick"]

_leading_whitespace_re = re.compile("(^[ \t]*)(?:[^ \t\n])", re.MULTILINE)
text = "\tfoo\n\tbar"
indents = _leading_whitespace_re.findall(text)
assert indents == ["\t", "\t"]

text = "  \thello there\n  \t  how are you?"
indents = _leading_whitespace_re.findall(text)
assert indents == ["  \t", "  \t  "]



# test PEP 526, variable annotations

x: int
print("x" in globals())

x: int = 1
print(x)

t: tuple = 1, 2
print(t)

# a pure annotation in a function makes that variable local
def f():
    x: int
    try:
        print(x)
    except NameError:
        print("NameError")
f()

# here, "x" should remain a global
def f():
    x.y: int
    print(x)
f()



# test bytearray.append method

a = bytearray(4)
print(a)

# append should append a single byte
a.append(2)
print(a)

# a should not be modified if append fails
try:
    a.append(None)
except TypeError:
    print('TypeError')
print(a)



# test basic async for execution
# example taken from PEP0492


class AsyncIteratorWrapper:
    def __init__(self, obj):
        print("init")
        self._obj = obj

    def __repr__(self):
        return "AsyncIteratorWrapper-" + self._obj

    def __aiter__(self):
        print("aiter")
        return AsyncIteratorWrapperIterator(self._obj)


class AsyncIteratorWrapperIterator:
    def __init__(self, obj):
        print("init")
        self._it = iter(obj)

    async def __anext__(self):
        print("anext")
        try:
            value = next(self._it)
        except StopIteration:
            raise StopAsyncIteration
        return value


def run_coro(c):
    print("== start ==")
    try:
        c.send(None)
    except StopIteration:
        print("== finish ==")


async def coro0():
    async for letter in AsyncIteratorWrapper("abc"):
        print(letter)


run_coro(coro0())


async def coro1():
    a = AsyncIteratorWrapper("def")
    async for letter in a:
        print(letter)
    print(a)


run_coro(coro1())

a_global = AsyncIteratorWrapper("ghi")


async def coro2():
    async for letter in a_global:
        print(letter)
    print(a_global)


run_coro(coro2())


async def coro3(a):
    async for letter in a:
        print(letter)
    print(a)


run_coro(coro3(AsyncIteratorWrapper("jkl")))



"""
The MIT License (MIT)

Copyright (c) 2023 Arduino SA

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

Basic example usage:

import time
from bmm150 import BMM150
from machine import Pin, SPI, I2C

# Init in I2C mode.
imu = BMM150(I2C(1, scl=Pin(15), sda=Pin(14)))

# Or init in SPI mode.
# TODO: Not supported yet.
# imu = BMM150(SPI(5), cs=Pin(10))

while (True):
    print('magnetometer: x:{:>8.3f} y:{:>8.3f} z:{:>8.3f}'.format(*imu.magnet()))
    time.sleep_ms(100)
"""

import array
import time
from micropython import const

_DEFAULT_ADDR = const(0x10)
_CHIP_ID = const(0x40)
_DATA = const(0x42)
_POWER = const(0x4B)
_OPMODE = const(0x4C)
_INT_STATUS = const(0x4A)
_TRIM_X1 = const(0x5D)
_TRIM_Y1 = const(0x5E)
_TRIM_Z4_LSB = const(0x62)
_TRIM_Z2_LSB = const(0x68)
_XYAXES_FLIP = const(-4096)
_ZHAXES_FLIP = const(-16384)
_ODR = const((10, 2, 6, 8, 15, 20, 25, 30))


class BMM150:
    def __init__(
        self,
        bus,
        cs=None,
        address=_DEFAULT_ADDR,
        magnet_odr=30,
    ):
        """Initalizes the Magnetometer.
        bus: IMU bus
        address: I2C address (in I2C mode).
        cs: SPI CS pin (in SPI mode).
        magnet_odr: (2, 6, 8, 10, 15, 20, 25, 30)
        """
        self.bus = bus
        self.cs = cs
        self.address = address
        self._use_i2c = hasattr(self.bus, "readfrom_mem")

        # Sanity checks
        if not self._use_i2c:
            raise ValueError("SPI mode is not supported")
        if magnet_odr not in _ODR:
            raise ValueError("Invalid sampling rate: %d" % magnet_odr)

        # Perform soft reset, and power on.
        self._write_reg(_POWER, 0x83)
        time.sleep_ms(10)

        if self._read_reg(_CHIP_ID) != 0x32:
            raise OSError("No BMM150 device was found at address 0x%x" % (self.address))

        # Configure the device.
        # ODR | OP: Normal mode
        self._write_reg(_OPMODE, _ODR.index(magnet_odr) << 3)

        # Read trim registers.
        trim_x1y1 = self._read_reg(_TRIM_X1, 2)
        trim_xyz_data = self._read_reg(_TRIM_Z4_LSB, 4)
        trim_xy1xy2 = self._read_reg(_TRIM_Z2_LSB, 10)

        self.trim_x1 = trim_x1y1[0]
        self.trim_y1 = trim_x1y1[1]
        self.trim_x2 = trim_xyz_data[2]
        self.trim_y2 = trim_xyz_data[3]
        self.trim_z1 = (trim_xy1xy2[3] << 8) | trim_xy1xy2[2]
        self.trim_z2 = (trim_xy1xy2[1] << 8) | trim_xy1xy2[0]
        self.trim_z3 = (trim_xy1xy2[7] << 8) | trim_xy1xy2[6]
        self.trim_z4 = (trim_xyz_data[1] << 8) | trim_xyz_data[0]
        self.trim_xy1 = trim_xy1xy2[9]
        self.trim_xy2 = trim_xy1xy2[8]
        self.trim_xyz1 = ((trim_xy1xy2[5] & 0x7F) << 8) | trim_xy1xy2[4]

        # Allocate scratch buffer.
        self.scratch = memoryview(array.array("h", [0, 0, 0, 0]))

    def _read_reg(self, reg, size=1):
        buf = self.bus.readfrom_mem(self.address, reg, size)
        if size == 1:
            return int(buf[0])
        return buf

    def _read_reg_into(self, reg, buf):
        self.bus.readfrom_mem_into(self.address, reg, buf)

    def _write_reg(self, reg, val):
        self.bus.writeto_mem(self.address, reg, bytes([val]))

    def _compensate_x(self, raw, hall):
        """Compensation equation ported from C driver"""
        x = 0
        if raw != _XYAXES_FLIP:
            x0 = self.trim_xyz1 * 16384 / hall
            x = x0 - 16384
            x1 = (self.trim_xy2) * (x**2 / 268435456)
            x2 = x1 + x * (self.trim_xy1) / 16384
            x3 = (self.trim_x2) + 160
            x4 = raw * ((x2 + 256) * x3)
            x = ((x4 / 8192) + (self.trim_x1 * 8)) / 16
        return x

    def _compensate_y(self, raw, hall):
        """Compensation equation ported from C driver"""
        y = 0
        if raw != _XYAXES_FLIP:
            y0 = self.trim_xyz1 * 16384 / hall
            y = y0 - 16384
            y1 = self.trim_xy2 * (y**2 / 268435456)
            y2 = y1 + y * self.trim_xy1 / 16384
            y3 = self.trim_y2 + 160
            y4 = raw * ((y2 + 256) * y3)
            y = ((y4 / 8192) + (self.trim_y1 * 8)) / 16
        return y

    def _compensate_z(self, raw, hall):
        """Compensation equation ported from C driver"""
        z = 0
        if raw != _ZHAXES_FLIP:
            z0 = raw - self.trim_z4
            z1 = hall - self.trim_xyz1
            z2 = self.trim_z3 * z1
            z3 = (self.trim_z1 * hall) / 32768
            z4 = self.trim_z2 + z3
            z5 = (z0 * 131072) - z2
            z = (z5 / (z4 * 4)) / 16
        return z

    def magnet_raw(self):
        for i in range(10):
            self._read_reg_into(_DATA, self.scratch)
            if self.scratch[3] & 0x1:
                return (
                    self.scratch[0] >> 3,
                    self.scratch[1] >> 3,
                    self.scratch[2] >> 1,
                    self.scratch[3] >> 2,
                )
            time.sleep_ms(30)
        raise OSError("Data not ready")

    def magnet(self):
        """Returns magnetometer vector."""
        x, y, z, h = self.magnet_raw()
        return (self._compensate_x(x, h), self._compensate_y(y, h), self._compensate_z(z, h))
